@@ -1,5 +1,5 @@
 # This file is part of CairoSVG
-# Copyright © 2010-2015 Kozea
+# Copyright © 2010-2018 Kozea
 #
 # This library is free software: you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -30,7 +30,8 @@ import tempfile
 import cairocffi as cairo
 import pytest
 
-from . import SURFACES, __version__, main, parser, surface, svg2pdf, svg2png
+from . import SURFACES, VERSION, parser, surface, svg2pdf, svg2png
+from .__main__ import main
 
 MAGIC_NUMBERS = {
     'SVG': b'<?xml',
@@ -180,14 +181,16 @@ def test_script():
         sys.stdin, sys.stdout = old_stdin, old_stdout
         return output
 
-    with tempfile.NamedTemporaryFile() as file_object:
+    with tempfile.NamedTemporaryFile(delete=False) as file_object:
         file_object.write(SVG_SAMPLE)
         file_object.flush()
         svg_filename = file_object.name
+        file_object.close()
 
         assert test_main(['--help'], exit_=True).startswith(b'usage: ')
         assert test_main(['--version'], exit_=True).strip() == (
-            __version__.encode('ascii'))
+            VERSION.encode('ascii'))
+
         assert test_main([svg_filename]) == expected_pdf
         assert test_main([svg_filename, '-d', '96', '-f', 'pdf']) == (
             expected_pdf)
@@ -219,3 +222,12 @@ def test_script():
             assert read_file(temp_3) == expected_pdf
         finally:
             shutil.rmtree(temp)
+
+        try:
+            os.remove(svg_filename)
+        except PermissionError:
+            # On Windows/NT systems, the temporary file sometimes fails to
+            # get deleted due to ``PermissionError`` exception. This is due
+            # to how Windows/NT handles the same file being opened twice at
+            # the same time.
+            pass
